@@ -22,11 +22,12 @@
 #    Boston, MA 02111-1307 USA
 
 # Standard modules
-import sys
-import select
-import struct
 import errno
+import math
+import select
 import socket
+import struct
+import sys
 
 # Python 2/3 compatibility.
 from six import PY3, byte2int, indexbytes
@@ -42,12 +43,36 @@ from . import rq
 from . import event
 
 if PY3:
-    def buffer(object, offset=None, size=None):
-        if offset is None:
-            offset = 0
+
+    class bytesview(object):
+
+        def __init__(self, data, offset=0, size=None):
+            if size is None:
+                size = len(data)-offset
+            if isinstance(data, bytes):
+                view = memoryview(data)
+            elif isinstance(data, bytesview):
+                view = data.view
+            else:
+                raise TypeError('unsupported type: {}'.format(type(data)))
+            self.view = view[offset:offset+size]
+
+        def __len__(self):
+            return len(self.view)
+
+        def __getitem__(self, key):
+            if isinstance(key, slice):
+                return bytes(self.view[key])
+            return self.view[key]
+
+else:
+
+    def bytesview(data, offset=0, size=None):
+        if not isinstance(data, (bytes, buffer)):
+            raise TypeError('unsupported type: {}'.format(type(data)))
         if size is None:
-            size = len(object)-offset
-        return memoryview(object)[offset:offset+size]
+            size = len(data)-offset
+        return buffer(data, offset, size)
 
 
 class Display(object):
@@ -96,6 +121,12 @@ class Display(object):
         self.request_waiting = 0
         self.request_wait_lock = lock.allocate_lock()
 
+        # Calculate optimal default buffer size for recv.
+        buffer_size = self.socket.getsockopt(socket.SOL_SOCKET,
+                                             socket.SO_RCVBUF)
+        buffer_size = math.pow(2, math.floor(math.log(buffer_size, 2)))
+        self.recv_buffer_size = int(buffer_size)
+
         # Data used by the send-and-recieve loop
         self.sent_requests = []
         self.recv_packet_len = 0
@@ -533,8 +564,12 @@ class Display(object):
             # Ignore errors caused by a signal recieved while blocking.
             # All other errors are re-raised.
             except select.error as err:
-                if err[0] != errno.EINTR:
-                    raise err
+                if isinstance(err, OSError):
+                    code = err.errno
+                else:
+                    code = err[0]
+                if code != errno.EINTR:
+                    raise
 
                 # We must lock send_and_recv before we can loop to
                 # the start of the loop
@@ -562,7 +597,9 @@ class Display(object):
                 # We're the recieving thread, parse the data
                 if recieving:
                     try:
-                        bytes_recv = self.socket.recv(2048)
+                        count = self.recv_packet_len - len(self.data_recv)
+                        count = max(self.recv_buffer_size, count)
+                        bytes_recv = self.socket.recv(count)
                     except socket.error as err:
                         self.close_internal('server: %s' % err[1])
                         raise self.socket_error
@@ -702,7 +739,7 @@ class Display(object):
         estruct = self.error_classes.get(code, error.XError)
 
         e = estruct(self, self.data_recv[:32])
-        self.data_recv = buffer(self.data_recv, 32)
+        self.data_recv = bytesview(self.data_recv, 32)
 
         # print 'recv Error:', e
 
@@ -756,7 +793,7 @@ class Display(object):
         req._parse_response(self.data_recv[:self.recv_packet_len])
         # print 'recv Request:', req
 
-        self.data_recv = buffer(self.data_recv, self.recv_packet_len)
+        self.data_recv = bytesview(self.data_recv, self.recv_packet_len)
         self.recv_packet_len = 0
 
 
@@ -793,7 +830,7 @@ class Display(object):
         if etype == ge.GenericEventCode:
             self.recv_packet_len = 0
 
-        self.data_recv = buffer(self.data_recv, length)
+        self.data_recv = bytesview(self.data_recv, length)
 
         # Drop all requests having an error handler,
         # but which obviously succeded.
